//===- PrivateManager.h - internal implementation of public manager -------===//
//
// LunarGLASS: An Open Modular Shader Compiler Architecture
// Copyright (C) 2010-2014 LunarG, Inc.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
//     Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
// 
//     Redistributions in binary form must reproduce the above
//     copyright notice, this list of conditions and the following
//     disclaimer in the documentation and/or other materials provided
//     with the distribution.
// 
//     Neither the name of LunarG Inc. nor the names of its
//     contributors may be used to endorse or promote products derived
//     from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
// COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
// ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
//===----------------------------------------------------------------------===//
//
// Author: John Kessenich, LunarG
//
// Private to LunarGLASS implementation, see LunarGLASSManager.h for public
// interface.  Inherit from gla::PrivateManager to customize your own manager.
//
//===----------------------------------------------------------------------===//

#pragma once
#ifndef PrivateManager_H
#define PrivateManager_H

// LunarGLASS includes
#include "LunarGLASSManager.h"

namespace gla {

    class BackEndTranslator;
    class BackEnd;

    class PrivateManager : public gla::Manager {
    public:
        virtual ~PrivateManager() { }
        virtual void clear() = 0;

        virtual const PrivateManager* getPrivateManager() const { return this; }

        virtual gla::BackEnd* const getBackEnd() const { return backEnd; }

        virtual void translateTopToBottom();
        virtual void translateBottomToTarget();
        virtual void dump(const char* heading);

    protected:
        PrivateManager() { }

        virtual void runLLVMOptimizations1();
        gla::BackEndTranslator* backEndTranslator;
        gla::BackEnd* backEnd;
    };
}

#endif /* PrivateManager_H */
